@@ -1,5 +1,5 @@
  /* fre:ac - free audio converter
-  * Copyright (C) 2001-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2001-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
 #include <jobs/job.h>
 #include <jobs/engine/convert.h>
 
-#include <gui/player.h>
+#include <gui/playback.h>
 #include <gui/edit_folder.h>
 
 #include <dialogs/config/configcomponent.h>
@@ -36,8 +36,6 @@ freac::LayerJoblist::LayerJoblist() : Layer("Joblist")
 {
 	BoCA::Config	*config = BoCA::Config::Get();
 
-	drawSurface		= NIL;
-
 	dontUpdateInfo		= False;
 
 	clicked_charset		= -1;
@@ -144,7 +142,7 @@ freac::LayerJoblist::LayerJoblist() : Layer("Joblist")
 		pos.x = 242;
 		pos.y = 0;
 
-		player	= new LayerPlayer(joblist);
+		player	= new PlaybackWidget(freacGUI::Get()->GetPlayer(), joblist);
 		player->SetPosition(pos);
 		player->SetOrientation(OR_UPPERRIGHT);
 	}
@@ -562,9 +560,7 @@ freac::LayerJoblist::LayerJoblist() : Layer("Joblist")
 	JobConvert::onFinishEncoding.Connect(&LayerJoblist::OnEncoderFinishEncoding, this);
 
 	JobConvert::onEncodeTrack.Connect(&LayerJoblist::OnEncoderEncodeTrack, this);
-
-	JobConvert::onTrackProgress.Connect(&LayerJoblist::OnEncoderTrackProgress, this);
-	JobConvert::onTotalProgress.Connect(&LayerJoblist::OnEncoderTotalProgress, this);
+	JobConvert::onReportProgress.Connect(&LayerJoblist::OnEncoderReportProgress, this);
 
 	/* Connect other slots.
 	 */
@@ -592,9 +588,7 @@ freac::LayerJoblist::~LayerJoblist()
 	JobConvert::onFinishEncoding.Disconnect(&LayerJoblist::OnEncoderFinishEncoding, this);
 
 	JobConvert::onEncodeTrack.Disconnect(&LayerJoblist::OnEncoderEncodeTrack, this);
-
-	JobConvert::onTrackProgress.Disconnect(&LayerJoblist::OnEncoderTrackProgress, this);
-	JobConvert::onTotalProgress.Disconnect(&LayerJoblist::OnEncoderTotalProgress, this);
+	JobConvert::onReportProgress.Disconnect(&LayerJoblist::OnEncoderReportProgress, this);
 
 	/* Clear tracks.
 	 */
@@ -714,17 +708,18 @@ Void freac::LayerJoblist::OnChangeSize(const Size &nSize)
 
 	/* Update title info area.
 	 */
-	info_edit_title->SetX(clientSize.cx - 170 - info_text_genre->GetUnscaledTextWidth() - info_text_year->GetUnscaledTextWidth() - info_edit_year->GetWidth() - info_edit_track->GetWidth());
-	info_edit_title->SetWidth(163 + info_text_genre->GetUnscaledTextWidth() + info_text_year->GetUnscaledTextWidth() + info_edit_year->GetWidth() + info_edit_track->GetWidth());
+	info_edit_title->SetX(Math::Min(clientSize.cx / 2 + 10 + (Int) Math::Max(info_text_title->GetUnscaledTextWidth(), info_text_track->GetUnscaledTextWidth()), clientSize.cx - 170 - info_text_genre->GetUnscaledTextWidth() - info_text_year->GetUnscaledTextWidth() - info_edit_year->GetWidth() - info_edit_track->GetWidth()));
+	info_edit_title->SetWidth(clientSize.cx - info_edit_title->GetX() - 7);
 	info_edit_track->SetX(info_edit_title->GetX());
 	info_text_year->SetX(info_edit_track->GetX() + info_edit_track->GetWidth() + 7);
 	info_edit_year->SetX(info_text_year->GetX() + info_text_year->GetUnscaledTextWidth() + 7);
 	info_text_genre->SetX(info_edit_year->GetX() + info_edit_year->GetWidth() + 7);
 	info_text_title->SetX(info_edit_title->GetX() - (Int) Math::Max(info_text_title->GetUnscaledTextWidth(), info_text_track->GetUnscaledTextWidth()) - 7);
 	info_text_track->SetX(info_text_title->GetX());
-	info_edit_artist->SetWidth(clientSize.cx - 21 - info_edit_artist->GetX() - (Int) Math::Max(info_text_title->GetUnscaledTextWidth(), info_text_track->GetUnscaledTextWidth()) - info_edit_title->GetWidth());
+	info_edit_artist->SetWidth(info_text_title->GetX() - info_edit_artist->GetX() - 7);
 	info_edit_album->SetWidth(info_edit_artist->GetWidth());
-	info_edit_genre->SetX(clientSize.cx - 142);
+	info_edit_genre->SetX(info_text_genre->GetX() + info_text_genre->GetWidth() + 7);
+	info_edit_genre->SetWidth(clientSize.cx - info_edit_genre->GetX() - 7);
 
 	htsp_edit_title->SetWidth(info_edit_title->GetWidth());
 	htsp_edit_artist->SetWidth(info_edit_artist->GetWidth());
@@ -735,16 +730,16 @@ Void freac::LayerJoblist::OnChangeSize(const Size &nSize)
 	Int	 maxTextSize = (Int) Math::Max(Math::Max(txt_progress->GetUnscaledTextWidth(), txt_outdir->GetUnscaledTextWidth()), Math::Max(txt_filename->GetUnscaledTextWidth(), txt_format->GetUnscaledTextWidth()));
 	Int	 maxTextSize2 = (Int) Math::Max(txt_filters->GetUnscaledTextWidth(), txt_encoder->GetUnscaledTextWidth());
 
-	txt_filters->SetX(clientSize.cx / 2 + maxTextSize2 - txt_filters->GetUnscaledTextWidth() + 7);
-	edb_filters->SetX(clientSize.cx / 2 + maxTextSize2 + 14);
+	txt_filters->SetX(clientSize.cx / 2 + maxTextSize2 - txt_filters->GetUnscaledTextWidth() + 3);
+	edb_filters->SetX(clientSize.cx / 2 + maxTextSize2 + 10);
 
-	txt_encoder->SetX(clientSize.cx / 2 + maxTextSize2 - txt_encoder->GetUnscaledTextWidth() + 7);
-	combo_encoder->SetX(clientSize.cx / 2 + maxTextSize2 + 14);
+	txt_encoder->SetX(clientSize.cx / 2 + maxTextSize2 - txt_encoder->GetUnscaledTextWidth() + 3);
+	combo_encoder->SetX(clientSize.cx / 2 + maxTextSize2 + 10);
 
-	edb_filename->SetWidth(clientSize.cx / 2 - 10 - maxTextSize - btn_skip->GetWidth());
-	edb_format->SetWidth(clientSize.cx / 2 - 14 - maxTextSize);
-	edb_filters->SetWidth(Math::Ceil(Float(clientSize.cx) / 2.0) - 44 - maxTextSize2);
-	combo_encoder->SetWidth(Math::Ceil(Float(clientSize.cx) / 2.0) - 44 - maxTextSize2);
+	edb_filename->SetWidth(clientSize.cx / 2 - 14 - maxTextSize - btn_skip->GetWidth());
+	edb_format->SetWidth(clientSize.cx / 2 - 18 - maxTextSize);
+	edb_filters->SetWidth(clientSize.cx - edb_filters->GetX() - btn_configure_dsp->GetWidth() - 3);
+	combo_encoder->SetWidth(edb_filters->GetWidth());
 	edb_outdir->SetWidth(clientSize.cx - 33 - maxTextSize - btn_browse->GetWidth() - btn_open->GetWidth());
 
 	btn_skip->SetX(edb_filename->GetX() + edb_filename->GetWidth());
@@ -941,8 +936,8 @@ Void freac::LayerJoblist::OnChangeLanguageSettings()
 	info_background->SetWidth(info_checkbox->GetUnscaledTextWidth() + 24);
 	info_background->Show();
 
-	info_edit_artist->SetX(info_text_artist->GetX() + (Int) Math::Max(info_text_artist->GetUnscaledTextWidth(), info_text_album->GetUnscaledTextWidth()) + 8);
-	info_edit_album->SetX(info_text_album->GetX() + (Int) Math::Max(info_text_artist->GetUnscaledTextWidth(), info_text_album->GetUnscaledTextWidth()) + 8);
+	info_edit_artist->SetX(info_text_artist->GetX() + (Int) Math::Max(info_text_artist->GetUnscaledTextWidth(), info_text_album->GetUnscaledTextWidth()) + 7);
+	info_edit_album->SetX(info_edit_artist->GetX());
 
 	/* OnChangeSize will correct sizes of any other widgets.
 	 */
@@ -967,8 +962,8 @@ Void freac::LayerJoblist::FillMenus()
 
 	if (Registry::Get().GetNumberOfComponentsOfType(COMPONENT_TYPE_OUTPUT) > 0)
 	{
-		menu_trackmenu->AddEntry(i18n->TranslateString("Play"))->onAction.Connect(&LayerPlayer::PlaySelectedItem, player);
-		menu_trackmenu->AddEntry(i18n->TranslateString("Stop"))->onAction.Connect(&LayerPlayer::StopPlayback, player);
+		menu_trackmenu->AddEntry(i18n->TranslateString("Play"))->onAction.Connect(&PlaybackWidget::PlaySelectedItem, player);
+		menu_trackmenu->AddEntry(i18n->TranslateString("Stop"))->onAction.Connect(&PlaybackWidget::StopPlayback, player);
 		menu_trackmenu->AddEntry();
 	}
 
@@ -1575,22 +1570,20 @@ Void freac::LayerJoblist::OnEncoderEncodeTrack(const Track &track, const String
 	surface->EndPaint();
 }
 
-Void freac::LayerJoblist::OnEncoderTrackProgress(Int progressValue, Int secondsLeft)
+Void freac::LayerJoblist::OnEncoderReportProgress(Int trackProgressValue, Int trackSecondsLeft, Int totalProgressValue, Int totalSecondsLeft)
 {
-	/* Start painting here, end it in OnEncoderTotalProgress.
-	 */
-	drawSurface = GetDrawSurface();
+	Surface	*surface = GetDrawSurface();
 
-	drawSurface->StartPaint(Rect::EncloseRect(progress->GetVisibleArea(), edb_totalTime->GetVisibleArea()));
+	surface->StartPaint(Rect::EncloseRect(progress->GetVisibleArea(), edb_totalTime->GetVisibleArea()));
 
 	/* Update seconds only if estimate is less or
 	 * at least two seconds more than before.
 	 */
-	if (secondsLeft < previousTrackSeconds || secondsLeft >= previousTrackSeconds + 2 || previousTrackSeconds == 0)
+	if (trackSecondsLeft < previousTrackSeconds || trackSecondsLeft >= previousTrackSeconds + 2 || previousTrackSeconds == 0)
 	{
 		/* Set track time string.
 		 */
-		String	 secondsString = Job::SecondsToString(secondsLeft);
+		String	 secondsString = Job::SecondsToString(trackSecondsLeft);
 
 		edb_trackTime->SetText(secondsString);
 
@@ -1601,29 +1594,14 @@ Void freac::LayerJoblist::OnEncoderTrackProgress(Int progressValue, Int secondsL
 			OnChangeSize(GetSize());
 		}
 
-		previousTrackSeconds = secondsLeft;
+		previousTrackSeconds = trackSecondsLeft;
 	}
 
-	/* Set percent values.
-	 */
-	if (progressValue > progress->GetValue())
-	{
-		edb_trackPercent->SetText(BoCA::I18n::Get()->TranslateString("%1%", "Technical").Replace("%1", String::FromInt(Math::Round(progressValue / 10.0))));
-
-		progress->SetValue(progressValue);
-	}
-}
-
-Void freac::LayerJoblist::OnEncoderTotalProgress(Int progressValue, Int secondsLeft)
-{
-	/* Update seconds only if estimate is less or
-	 * at least two seconds more than before.
-	 */
-	if (secondsLeft < previousTotalSeconds || secondsLeft >= previousTotalSeconds + 2 || previousTotalSeconds == 0)
+	if (totalSecondsLeft < previousTotalSeconds || totalSecondsLeft >= previousTotalSeconds + 2 || previousTotalSeconds == 0)
 	{
 		/* Set total time string.
 		 */
-		String	 secondsString = Job::SecondsToString(secondsLeft);
+		String	 secondsString = Job::SecondsToString(totalSecondsLeft);
 
 		edb_totalTime->SetText(secondsString);
 
@@ -1634,21 +1612,26 @@ Void freac::LayerJoblist::OnEncoderTotalProgress(Int progressValue, Int secondsL
 			OnChangeSize(GetSize());
 		}
 
-		previousTotalSeconds = secondsLeft;
+		previousTotalSeconds = totalSecondsLeft;
 	}
 
 	/* Set percent values.
 	 */
-	if (progressValue > progress_total->GetValue())
+	if (trackProgressValue > progress->GetValue())
 	{
-		edb_totalPercent->SetText(BoCA::I18n::Get()->TranslateString("%1%", "Technical").Replace("%1", String::FromInt(Math::Round(progressValue / 10.0))));
+		edb_trackPercent->SetText(BoCA::I18n::Get()->TranslateString("%1%", "Technical").Replace("%1", String::FromInt(Math::Round(trackProgressValue / 10.0))));
 
-		progress_total->SetValue(progressValue);
+		progress->SetValue(trackProgressValue);
 	}
 
-	/* End painting, started in OnEncoderTrackProgress.
-	 */
-	drawSurface->EndPaint();
+	if (totalProgressValue > progress_total->GetValue())
+	{
+		edb_totalPercent->SetText(BoCA::I18n::Get()->TranslateString("%1%", "Technical").Replace("%1", String::FromInt(Math::Round(totalProgressValue / 10.0))));
+
+		progress_total->SetValue(totalProgressValue);
+	}
+
+	surface->EndPaint();
 }
 
 Void freac::LayerJoblist::ShowHideTitleInfo()
